@@ -40,8 +40,10 @@ sub new {
   not $self->{from}{path}{$path}
     or croak "$self->{from}{class} already has a path '$path'";
   $self->{from}{path}{$path} = $self;
+
+  # if this is a composition path, remember it in the 'components' array
   push @{$self->{from}{components}}, $path
-    if $self->{association}{kind} eq 'Composition';
+    if $self->{association}{kind} eq 'Composition' && $self->{multiplicity}[1] > 1;
 
   # install a navigation method into the 'from' table class
   my @navigation_args = ($self->{name},  # method name
